@@ -3,7 +3,6 @@ author: Maikel Punie <maikel.punie@gmail.com>
 """
 from __future__ import annotations
 
-import json
 import string
 
 from velbusaio.command_registry import commandRegistry
@@ -11,15 +10,10 @@ from velbusaio.const import (
     DEVICE_CLASS_ILLUMINANCE,
     DEVICE_CLASS_TEMPERATURE,
     ENERGY_KILO_WATT_HOUR,
-    ENERGY_WATT_HOUR,
     TEMP_CELSIUS,
-    VOLUME_CUBIC_METER,
     VOLUME_CUBIC_METER_HOUR,
-    VOLUME_LITERS,
     VOLUME_LITERS_HOUR,
 )
-from velbusaio.messages.switch_relay_off import SwitchRelayOffMessage
-from velbusaio.messages.switch_relay_on import SwitchRelayOnMessage
 
 
 class Channel:
@@ -50,8 +44,16 @@ class Channel:
     def get_module_serial(self) -> str:
         return self._module.get_serial()
 
-    def get_module_address(self) -> int:
-        return self._module._address
+    def get_module_address(self, chan_type="") -> int:
+        """Return (sub)module address for channel"""
+        _mod_address = self._address
+        if chan_type == "Button" and self._num > 8:
+            _mod_address = self._module.get_addresses()[1]
+            if self._num > 16:
+                _mod_address = self._module.get_addresses()[2]
+                if self._num > 24:
+                    _mod_address = self._module.get_addresses()[3]
+        return _mod_address
 
     def get_module_sw_version(self) -> str:
         return self._module.get_sw_version()
@@ -128,14 +130,16 @@ class Channel:
         """
         Set the attributes of this channel
         """
-        for key, val in data.items():
-            setattr(self, f"_{key}", val)
-        for m in self._on_status_update:
-            await m()
+        for key, new_val in data.items():
+            cur_val = getattr(self, f"_{key}", None)
+            if cur_val is None or cur_val != new_val:
+                setattr(self, f"_{key}", new_val)
+                for m in self._on_status_update:
+                    await m()
 
     def get_categories(self) -> list:
         """
-        Get the categories (for hass)
+        Get the categories (mainly for home-assistant)
         """
         # COMPONENT_TYPES = ["switch", "sensor", "binary_sensor", "cover", "climate", "light"]
         return []
@@ -147,7 +151,6 @@ class Channel:
 class Blind(Channel):
     """
     A blind channel
-    HASS OK
     """
 
     _state = None
@@ -204,7 +207,6 @@ class Blind(Channel):
 class Button(Channel):
     """
     A Button channel
-    HASS OK
     """
 
     _enabled = True
@@ -212,7 +214,9 @@ class Button(Channel):
     _led_state = None
 
     def get_categories(self) -> list:
-        return ["binary_sensor", "led"]
+        if self._enabled:
+            return ["binary_sensor", "led"]
+        return []
 
     def is_closed(self) -> bool:
         """
@@ -244,9 +248,12 @@ class Button(Channel):
             code = 0xF5
         else:
             return
+
+        _mod_add = self.get_module_address("Button")
+        _chn_num = self._num - self._module.calc_channel_offset(_mod_add)
         cls = commandRegistry.get_command(code, self._module.get_type())
-        msg = cls(self._address)
-        msg.leds = [self._num]
+        msg = cls(_mod_add)
+        msg.leds = [_chn_num]
         await self._writer(msg)
         await self.update({"led_state": state})
 
@@ -255,7 +262,6 @@ class ButtonCounter(Button):
     """
     A ButtonCounter channel
     This channel can act as a button and as a counter
-    HASS OK
     """
 
     _Unit = None
@@ -277,7 +283,7 @@ class ButtonCounter(Button):
         val = 0
         # if we don't know the delay
         # or we don't know the unit
-        # or the daly is the max value
+        # or the delay is the max value
         #   we always return 0
         if not self._delay or not self._Unit or self._delay == 0xFFFF:
             return round(0, 2)
@@ -292,20 +298,25 @@ class ButtonCounter(Button):
         return round(val, 2)
 
     def get_unit(self) -> str:
-        return "W"
+        if self._Unit in (
+            VOLUME_LITERS_HOUR,
+            VOLUME_CUBIC_METER_HOUR,
+            ENERGY_KILO_WATT_HOUR,
+        ):
+            return self._Unit
+        return None
 
     def get_counter_state(self) -> int:
         return round((self._counter / self._pulses), 2)
 
     def get_counter_unit(self) -> str:
-        return ENERGY_KILO_WATT_HOUR
+        return self._Unit
 
 
 class Sensor(Button):
     """
     A Sensor channel
-    HASS OK
-    This is a bit wierd, but this happens because of code sharing with openhab
+    This is a bit weird, but this happens because of code sharing with openhab
     A sensor in this case is actually a Button
     """
 
@@ -314,14 +325,12 @@ class ThermostatChannel(Button):
     """
     A Thermostat channel
     These are the booster/heater/alarms
-    HASS OK
     """
 
 
 class Dimmer(Channel):
     """
     A Dimmer channel
-    HASS OK
     """
 
     _state: int = 0
@@ -368,7 +377,6 @@ class Dimmer(Channel):
 class Temperature(Channel):
     """
     A Temperature sensor channel
-    HASS OK
     """
 
     _cur = 0
@@ -390,11 +398,20 @@ class Temperature(Channel):
     def is_temperature(self) -> bool:
         return True
 
+    def get_max(self) -> int:
+        if self._max is None:
+            return None
+        return round(self._max, 2)
+
+    def get_min(self) -> int:
+        if self._min is None:
+            return None
+        return round(self._min, 2)
+
 
 class SensorNumber(Channel):
     """
     A Numeric Sensor channel
-    HASS OK
     """
 
     _cur = 0
@@ -415,7 +432,6 @@ class SensorNumber(Channel):
 class LightSensor(Channel):
     """
     A light sensor channel
-    HASS OK
     """
 
     _cur = 0
@@ -436,13 +452,18 @@ class LightSensor(Channel):
 class Relay(Channel):
     """
     A Relay channel
-    HASS OK
     """
 
     _on = None
+    _enabled = True
+    _inhibit = False
+    _forced_on = False
+    _disabled = False
 
     def get_categories(self) -> list:
-        return ["switch"]
+        if self._enabled:
+            return ["switch"]
+        return []
 
     def is_on(self) -> bool:
         """
@@ -450,6 +471,15 @@ class Relay(Channel):
         """
         return self._on
 
+    def is_inhibit(self) -> bool:
+        return self._inhibit
+
+    def is_forced_on(self) -> bool:
+        return self._forced_on
+
+    def is_disabled(self) -> bool:
+        return self._disabled
+
     async def turn_on(self) -> None:
         """
         Send the turn on message
@@ -483,6 +513,19 @@ class Memo(Channel):
     A Memo text
     """
 
+    async def set(self, txt: str) -> None:
+        cls = commandRegistry.get_command(0xAC, self._module.get_type())
+        msg = cls(self._address)
+        msgcntr = 0
+        for char in txt:
+            msg.memo_text += char
+            if len(msg.memo_text) >= 5:
+                msgcntr += 5
+                await self._writer(msg)
+                msg = cls(self._address)
+                msg.start = msgcntr
+        await self._writer(msg)
+
 
 # _mode = None
 # _target = None
